#!/usr/bin/env python
#
# Prelude-Trigger, (C) 2009 Alexander Tsvyashchenko <prelude@ndl.kiev.ua>
# Based on Prelude-Correlator, Copyright (C) 2009 PreludeIDS Technologies.
# Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

import require
import sys, os, time, signal
import yaml
from optparse import OptionParser
from PreludeEasy import ClientEasy, CheckVersion
from PreludeTrigger import __version__ as VERSION
from PreludeTrigger import idmef, pluginmanager, log


LIBPRELUDE_REQUIRED_VERSION = "0.9.23"


class Env:
        def __init__(self, conf_filename, dry_run):
                self.logger = log.Log(os.path.join(os.path.dirname(conf_filename), os.path.basename(conf_filename), ".conf"))
		stream = file(conf_filename, 'r')
                self.config = yaml.load(stream)
		stream.close()
		self.dry_run = dry_run
                self.pluginmanager = pluginmanager.PluginManager(self)

                self.logger.info("%d filter plugins have been loaded." % (self.pluginmanager.getFilterPluginCount()))
                self.logger.info("%d action plugins have been loaded." % (self.pluginmanager.getActionPluginCount()))


class SignalHandler:
        def __init__(self, env):
                self._env = env
                signal.signal(signal.SIGTERM, self._handle_signal)
                signal.signal(signal.SIGINT, self._handle_signal)
                signal.signal(signal.SIGQUIT, self._handle_signal)

        def _handle_signal(self, signum, frame):
                self._env.logger.info("caught signal %d" % signum)

                if signum == signal.SIGQUIT:
                        self._env.prelude_client.stats()
                else:
                        self._env.prelude_client.stop()


class PreludeClient:
        def __init__(self, env, print_input=None):
                self._env = env
                self._message_processed = 0
                self._trigger_executed = 0
                self._print_input = print_input
                self._continue = True

		self._client = ClientEasy("prelude-trigger", ClientEasy.PERMISSION_IDMEF_READ,
                                          "Prelude-Trigger", "Trigger", "Alexander Tsvyashchenko",
                                          VERSION)
                self._client.Start()


        def _handle_event(self, idmef):
                if self._print_input:
                    self._print_input.write(str(idmef))

                for trigger in self._env.config['triggers']:
		    if self._env.pluginmanager.run(trigger, idmef):
                        self._trigger_executed = self._trigger_executed + 1
                        self._env.logger.debug("executed trigger %s for event %s" % (trigger['name'], str(idmef)))

                self._message_processed += 1

        def stats(self):
                self._env.logger.info("%d message received, %d triggers executed." % (self._message_processed, self._trigger_executed))

        def recvEvent(self):
                msg = idmef.IDMEF()

                while self._continue:
                        try:
                            r = self._client.RecvIDMEF(msg)
                        except:
                                r = 0

                        if r:
                            if msg.Get("alert.create_time"):
                                self._handle_event(msg)
                            msg.reset()

        def stop(self):
                self._continue = False


def main():
        if not CheckVersion(LIBPRELUDE_REQUIRED_VERSION):
                raise Exception, ("Libprelude version '%s' is required" % LIBPRELUDE_REQUIRED_VERSION)

        config_filename = require.get_config_filename(None, "prelude-trigger.yaml")

        parser = OptionParser(usage="%prog", version="%prog " + VERSION)
        parser.add_option("-c", "--config", action="store", dest="config", type="string", help="Configuration file to use", metavar="FILE", default=config_filename)
        parser.add_option("", "--dry-run", action="store_true", dest="dry_run", help="Triggers will be logged but not executed", default=False)
        parser.add_option("-d", "--daemon", action="store_true", dest="daemon", help="Run in daemon mode")
        parser.add_option("-P", "--pidfile", action="store", dest="pidfile", type="string", help="Write Prelude Trigger PID to specified file", metavar="FILE")
        parser.add_option("", "--print-input", action="store", dest="print_input", type="string", help="Dump alert input from manager to the specified file", metavar="FILE")
        parser.add_option("--debug", action="store", dest="debug", type="int", help="Enable debug ouptut (optional debug level argument)", metavar="LEVEL")
        (options, args) = parser.parse_args()

        env = Env(options.config, options.dry_run)

        ifd = None
        if options.print_input:
                if options.print_input == "-":
                        ifd = sys.stdout
                else:
                        ifd = open(options.print_input, "w")

        if options.daemon:
            if os.fork():
                os._exit(0)

            os.setsid()
            if os.fork():
                os._exit(0)

            os.umask(077)

            fd = os.open('/dev/null', os.O_RDWR)
            for i in xrange(3):
                os.dup2(fd, i)

            os.close(fd)
            if options.pidfile:
                open(options.pidfile, "w").write(str(os.getpid()))

        env.prelude_client = PreludeClient(env, print_input=ifd)
        idmef.set_prelude_client(env.prelude_client)

        SignalHandler(env)

        env.prelude_client.recvEvent()
